
/* Texture analysis using the co-occurrence matrix
   and the autocorrelation function.
*/


/*		MODULE FOR CRYSTAL IMAGE 0.85 BETA
		This program is copyrighted material.
		(C) 2002-2010 by Mark A. Haidekker, all rights reserved.
		
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE:

		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
		APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
		HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
		OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
		THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
		PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
		IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
		ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
		WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
		THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
		GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
		USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
		DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
		PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
		EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
		SUCH DAMAGES.

		For the full text of the license, please see the file COPYING
		in the project's main directory, or read it on the Web at
		http://www.gnu.org/copyleft/gpl.html

 */




#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>



#include "../src/defs.h"
#include "../src/dlgbox.h"
#include "../src/functions.h"
#include "../src/findclust.h"
#include "../src/interface.h"
#include "../src/graphdsp.h"
#include "../src/dso.h"
#include "../src/macro.h"
#include "../src/gtkhelpers.h"
#include "../src/special1.h"
#include "../src/imgbuf.h"
#include "../src/application.h"
#include "../src/secondwnd.h"
#include "../src/tiffstuff.h"
#include "../src/roi.h"
#include "../src/viz.h"


typedef struct
{
	float avx, avy, sdx, sdy;
	float energy, inertia, correl, id, idm;
	float entropy, contrast, clustering;
}
haralick_type;




/*  #*#     #*#             #*#     #*#     #*#    #   #   ####    ####    #####   #   #    #*#    #####   */
/* #   #   #   #           #   #   #   #   #   #   #   #   #   #   #   #   #       #   #   #   #   #       */
/* #       #   #           #   #   #       #       #   #   #   #   #   #   #       ##  #   #       #       */
/* #       #   #   #####   #   #   #       #       #   #   ####    ####    ####    # # #   #       ####    */
/* #       #   #           #   #   #       #       #   #   # #     # #     #       #  ##   #       #       */
/* #   #   #   #           #   #   #   #   #   #   #   #   #  #    #  #    #       #   #   #   #   #       */
/*  #*#     #*#             #*#     #*#     #*#     #*#    #   #   #   #   #####   #   #    #*#    #####   */



void compute_cooccurrence (image_type *img, int slice, int numbins, int xshift, int yshift)
{
image_type glcm;				/* Gray-level co-occurrence matrix */
float *p;
int bx1,bx2,by1,by2;			/* ROI's bounding box */
int x,y,x1,y1,x2,y2,cnt;
float buf, min, max, delta;


	if (allocate_image (&glcm, B_FLOAT, numbins, numbins, 1)==-1)
		fatal_memerr();
	p = glcm.data;

	roi_bbox (img, &bx1, &by1, &bx2, &by2);			/* Observe ROI on input image */

	/* Bin size and offset */
	
	imgminmax (img, &min, &max);
	delta = (max-min)/(numbins-1);
	//min=0; delta=1;		/* Force for 8-bit image */
	cnt=0;

	/* Fill the glc matrix. Observe ROI. One slice only. Use luminance in RGB (ugh. another one of those). */

	for (y=by1; y<=by2; y++)
		for (x=bx1; x<=bx2; x++)
		{
			if (in_roi(x,y))
			{
		
				buf = readbuf_flt (img,x,y,slice);
				x1 = (buf-min)/delta;
				x2=x+xshift; y2=y+yshift;
				if ((x2>=0) && (y2>=0) && (x2<img->xmax) && (y2<img->ymax))
				{
					buf = readbuf_flt (img,x2,y2,slice);
					y1 = (buf-min)/delta;
					if ((x1>=0) && (y1>=0) && (x1<numbins) && (y1<numbins))
					{
						p[x1+numbins*y1]+=1.0;
						cnt++;
					}
					else
					{
						dp (1, "GLCM: Illegal bin at (%d,%d) glcm coords (%d,%d)\n",x,y,x1,y1);
					}
				}
			}
		}

	/* Normalize by cnt to obtain the probability function rhather than the counts */
	
	for (y=0; y<numbins; y++)
		for (x=0; x<numbins; x++)
		{
			p[x+numbins*y] /= cnt;
		}

	make_minmax (&glcm);
	handback (img, &glcm);

}


/* A similar theme. cooccurrence2 needs two images of equal dimension (but not
	necessarily equal types). It computes the GLCM of these two images without
	any shift. Used, for example, in registration.
	This operation is fully 3D.
	Another difference is that this function returns an image pointer
	rather than using handback.
*/

image_type compute_cooccurrence2 (image_type *img1, image_type *img2, int numbins)
{
image_type glcm;				/* Gray-level co-occurrence matrix */
float *p;
int x,y,z,x1,y1,cnt;
float buf, min,min1,min2, max,max1,max2, delta;

	/* Before everything else, check matching size */

	glcm.data = NULL;
	if ((img1->xmax != img2->xmax) || (img1->ymax != img2->ymax) || (img1->zmax != img2->zmax))
		return glcm;

	if (allocate_image (&glcm, B_FLOAT, numbins, numbins, 1)==-1)
		fatal_memerr();
	p = glcm.data;

	/* Bin size and offset */
	
	imgminmax (img1, &min1, &max1);
	imgminmax (img2, &min2, &max2);
	min=MIN(min1,min2);
	max=MAX(max1,max2);
	delta = (max-min)/(numbins-1);
	//min=0; delta=1;		/* Force for 8-bit image */
	cnt=0;

	/* Fill the glc matrix. Observe ROI.Full 3D, all slices. Use luminance in RGB (ugh. another one of those). */

	for (z=0; z<img1->zmax; z++)
		for (y=0; y<img1->ymax; y++)
			for (x=0; x<img1->xmax; x++)
			{
				buf = readbuf_flt (img1,x,y,z);
				x1 = (buf-min)/delta;
				buf = readbuf_flt (img2,x,y,z);
				y1 = (buf-min)/delta;
				if ((x1>=0) && (y1>=0) && (x1<numbins) && (y1<numbins))
				{
					p[x1+numbins*y1]+=1.0;
					cnt++;
				}
				else
				{
					dp (1, "GLCM: Illegal bin at (%d,%d) glcm coords (%d,%d)\n",x,y,x1,y1);
				}

			}

	/* Normalize by cnt to obtain the probability function rhather than the counts */
	
	for (y=0; y<numbins; y++)
		for (x=0; x<numbins; x++)
		{
			p[x+numbins*y] /= cnt;
		}

	make_minmax (&glcm);
	glcm.uid = next_uid++;
	return glcm;
}





/* Compute some of Haralick's texture metrics from the 
	co-occurrence matrix passed in glcm.
	glcm is assumed to be 2D. */

haralick_type co_occurrence_metrix (image_type *glcm)
{
int x,y;
float buf;
float avx, avy, sdx, sdy;		/* Average and std dev in x and y direction */
float energy, inertia, correl, id, idm;
float entropy, contrast, clustering;
haralick_type result;



	/* Need to compute average first */

	avx=0.0; avy=0.0;
	for (y=0; y<glcm->xmax; y++)
		for (x=0; x<glcm->xmax; x++)
		{
			buf = readbuf_flt (glcm,x,y,0);
			avx += x*buf;
			avy += y*buf;
		}

	/* Next, compute standard deviations and several metrics */

	sdx=0.0; sdy=0.0; energy=0.0; inertia=0.0; 
	correl=0.0; id=0.0; idm=0.0; entropy=0.0;
	contrast=0.0; clustering=0.0;
	for (y=0; y<glcm->xmax; y++)
		for (x=0; x<glcm->xmax; x++)
		{
			buf = readbuf_flt (glcm,x,y,0);
			sdx += SQR(x-avx)*buf;
			sdy += SQR(y-avy)*buf;
			energy += SQR(buf);
			inertia += SQR(x-y)*buf;
			contrast += buf*abs(x-y);
			/* correl += (x-avx)*(y-avy)*buf;		According to Haralick, see next line for improvement */
			correl += x*y*buf - avx*avy;
			if (x!=y) id += buf/abs(x-y);	/* Inverse difference */
			idm += buf/(1+SQR(x-y));		/* Inverse difference moment */
			if (buf>1e-10) entropy -= buf*SAFELOG(buf);
			clustering += SQR(x-avx+y-avy)*buf;
		}

	sdx=sqrt(sdx); sdy=sqrt(sdy);
	correl /= (sdx*sdy);
	entropy *= 1.442695;		/* log -> log2 */

	/* Prepare for easy function return */

	result.avx=avx; result.avy=avy;
	result.sdx=sdx; result.sdy=sdy;
	result.energy=energy;
	result.inertia=inertia;
	result.correl= correl;
	result.id= id;
	result.idm= idm;
	result.entropy=entropy;
	result.contrast=contrast;
	result.clustering=clustering;

	return result;

}



/* Compute the co-occurrence matrix. Do this by first cloning
	the current slice of mainimage and submitting it to compute_cooccurrence.
	Then display the result in the secondary window.
	No undo needed here since we don't touch the main image */

void cooccurrence_main (int numbins, int xshift, int yshift)
{
image_type glcm;
char report[16384]; char s[256];
haralick_type h;

	if (allocate_image (&glcm, mainimg.imgtype, mainimg.xmax, mainimg.ymax, 1)==-1)
		fatal_memerr();
	copybuf_z (&mainimg, &glcm, cur_slice, cur_slice);

	compute_cooccurrence (&glcm, 0, numbins, xshift, yshift);
	h = co_occurrence_metrix (&glcm);
	secondary_display (&glcm, 0, "GLCM");
	freebuf (&glcm);
	
	sprintf (report, "Texture feature metrics\n\n");
	sprintf (s, "Energy: %f\n",h.energy); strncat (report, s, 16000);
	sprintf (s, "Entropy: %f\n",h.entropy); strncat (report, s, 16000);
	sprintf (s, "Inertia: %f\n",h.inertia); strncat (report, s, 16000);
	sprintf (s, "Correlation: %f\n",h.correl); strncat (report, s, 16000);
	sprintf (s, "Contrast: %f\n",h.contrast); strncat (report, s, 16000);
	sprintf (s, "Clustering: %f\n",h.clustering); strncat (report, s, 16000);
	sprintf (s, "Inv difference: %f\n",h.id); strncat (report, s, 16000);
	sprintf (s, "Inv difference moment: %f\n",h.idm); strncat (report, s, 16000);
	ShowLongMessage (report, "Texture Features");

}




/********************************************************************************/


GtkAdjustment *cooc_yshift_adj, *cooc_numbins_adj, *cooc_xshift_adj;


void on_cooc_okbutton_clicked (GtkButton *button, gpointer user_data)
{
int xshift, yshift;
int numbins;

	numbins = cooc_numbins_adj->value;
	xshift = cooc_xshift_adj->value;
	yshift = cooc_yshift_adj->value;

	cooccurrence_main (numbins, xshift, yshift);
	
	gtk_widget_destroy (GTK_WIDGET(user_data));

}




void on_cooc_cancelbutton_clicked (GtkButton *button, gpointer user_data)
{

	gtk_widget_destroy (GTK_WIDGET(user_data));

}






void create_cooc_window (const gchar* title)
{
  GtkWidget *cooc_window;
  GtkWidget *cooc_table;
  GtkWidget *cooc_hbox, *cooc_vbox;
  GtkWidget *cooc_label1;
  GtkWidget *cooc_label2;
  GtkWidget *cooc_label3;
  GtkWidget *cooc_label4;
  GtkWidget *cooc_label5;
  GtkWidget *cooc_hseparator1;
  GtkWidget *cooc_okbutton, *cooc_cancelbutton;
  GtkWidget *cooc_numbins_hscale;
  GtkWidget *cooc_xshift_hscale;
  GtkWidget *cooc_yshift_hscale;
  int y;


	cooc_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (cooc_window), "cooc_window", cooc_window);
	if (title==NULL)
		gtk_window_set_title (GTK_WINDOW (cooc_window), "Co-Occurrence Matrix");
	else
		gtk_window_set_title (GTK_WINDOW (cooc_window), title);

	cooc_vbox = vbox_in_window (cooc_window, NULL);
	cooc_table= MakeTable (cooc_window, cooc_vbox, NULL, 2,6);
	y=0;


	
	cooc_label1 = MakeLabel (cooc_window,cooc_table,"No. of bins","cooc_label1",0,y);
	cooc_numbins_adj = GTK_ADJUSTMENT(gtk_adjustment_new (256, 10, 256, 1, 16, 0));
	cooc_numbins_hscale = MakeHScale (cooc_numbins_adj, cooc_window, cooc_table, "cooc_numbins_hscale",1,y);
	gtk_scale_set_digits (GTK_SCALE (cooc_numbins_hscale), 0);
	y++;

	cooc_label2 = MakeLabel (cooc_window,cooc_table,"Shift X","cooc_label2",0,y);
	cooc_xshift_adj = GTK_ADJUSTMENT(gtk_adjustment_new (1, -15, 15, 1, 1, 0));
	cooc_xshift_hscale = MakeHScale (cooc_xshift_adj, cooc_window, cooc_table, "cooc_xshift_hscale",1,y);
	gtk_scale_set_digits (GTK_SCALE (cooc_xshift_hscale), 0);
	y++;

	cooc_label3 = MakeLabel (cooc_window,cooc_table,"Shift Y","cooc_label3",0,y);
	cooc_yshift_adj = GTK_ADJUSTMENT(gtk_adjustment_new (0, -15, 15, 1, 1, 0));
	cooc_yshift_hscale = MakeHScale (cooc_yshift_adj, cooc_window, cooc_table, "cooc_yshift_hscale",1,y);
	gtk_scale_set_digits (GTK_SCALE (cooc_yshift_hscale), 0);
	y++;

	cooc_hseparator1 = MakeHsep (cooc_window, cooc_table, NULL, 0,y);
	cooc_hseparator1 = MakeHsep (cooc_window, cooc_table, NULL, 1,y);
	y++;
	
	cooc_hbox = hbox_in_window (cooc_window, cooc_vbox, NULL);

	cooc_okbutton = Button_in_table (cooc_window, cooc_hbox, "OK", "cooc_okbutton", -1,-1, on_cooc_okbutton_clicked);
	cooc_cancelbutton = Button_in_table (cooc_window, cooc_hbox, "Cancel", "cooc_cancelbutton", -1,-1, NULL);

	gtk_signal_connect (GTK_OBJECT (cooc_window), "destroy",
                    	GTK_SIGNAL_FUNC (on_cooc_cancelbutton_clicked), cooc_window);
	gtk_signal_connect (GTK_OBJECT (cooc_cancelbutton), "clicked",
                    	GTK_SIGNAL_FUNC (on_cooc_cancelbutton_clicked), cooc_window);

	gtk_widget_show (cooc_window);
}



void cooc_main ()
{

	undo_nexus (&mainimg);
	create_cooc_window ("Co-Occurrence Matrix");


}



/*
	GLCM function:
	Syntax 1: imgvar = GLCM (imgexpr, slice, numbins, xshift, yshift);
		Create the co-occurrence matrix of an image (single-slice operation on slice "slice")
		Numbins: Number of bins and at the same time size of (square) output image
		xshift, yshift: Displacement vector, more commonly known as displacement d and angle theta
	Syntax 2: imgvar = GLCM (imgexpr1, imgexpr2, numbins);
		Create the co-occurrence matrix of two images that need to have identical size,
		imgexpr1 and imgexpr2.
		Numbins: Number of bins and at the same time size of (square) output image

*/

variable_type mac_glcm (unsigned char* s, int* i)
{
variable_type v1,v2,r;
image_type v;
int j,k;
int slice, numbins, xshift, yshift;


	/* The image stack */

	j = *i; k=0; v1 = nullvar; v2=nullvar;
	v1 = expr (s,&j, MT_IMG);		/* Get image (or first image) to compute GLCM of */

	/* Now we need to differentiate between the first and second syntax.
		Do we have an image or a numvalue? */

	if (s[j++]!=',') macro_error (1,j);
	v2 = expr (s,&j, MT_NULL);			/* Allow any return type */

	if (v2.type==MT_IMG)				/* Syntax 2 */
	{
		if (s[j++]!=',') macro_error (1,j);
		r = expr (s,&j, MT_REAL); numbins = r.intvalue;
		while (s[j]!=')') j++;
		*i = j+1;

		v = compute_cooccurrence2 (&v1.imgptr, &v2.imgptr, numbins);

		/* Dispose of v1 and v2, then return image v in variable v1 */

		freebuf (&v1.imgptr); freebuf (&v2.imgptr);
		v1.type=MT_IMG;
		v1.imgptr = v;

		return v1;


	}
	else if (v2.type==MT_INT)			/* Syntax 1 */
	{
		/* The remaining parameters (all are integer) */	
	
		slice = v2.intvalue;
		if (s[j++]!=',') macro_error (1,j);
		r = expr (s,&j, MT_REAL); numbins = r.intvalue;
		if (s[j++]!=',') macro_error (1,j);
		r = expr (s,&j, MT_REAL); xshift = r.intvalue;
		if (s[j++]!=',') macro_error (1,j);
		r = expr (s,&j, MT_REAL); yshift = r.intvalue;
		while (s[j]!=')') j++;
		*i = j+1;

		compute_cooccurrence (&v1.imgptr, slice, numbins, xshift, yshift);
		v1.type=MT_IMG;
		return v1;
	}
	else
		macro_error (6,j);				/* This return type is not allowed */

	return nullvar;			/* This point is never reached */

}


/*
	GLCMVECTOR function:
	Syntax: numvar = GLCMVECTOR (imgexpr);
	Compute the feature vector from the co-occurrence matrix provided in imgexpr.
	This function returns the energy (arbitrarily chosen)
	and stores the feature vector in the user[] array. The sequence from user[0]
	to user[7] is: Energy, Entropy, Correlation, Inverse Difference, Inverse
	Difference Moment, Contrast, Clustering, Inertia.
}

*/

variable_type mac_glcmvector (unsigned char* s, int* i)
{
variable_type v,r;
int j,k;
haralick_type h;


	/* The image stack */

	j = *i; k=0; v = nullvar;
	v = expr (s,&j, MT_IMG);		/* Get GLCM image */
	
	while (s[j]!=')') j++;
	*i = j+1;

	/* Go execute. Prepare r to be returned as real value */

	h = co_occurrence_metrix (&v.imgptr);
	freebuf (&v.imgptr);
	r = nullvar;
	r.type=MT_REAL;
	r.numvalue = h.energy;

	/* Store feature vector in user[] */

	mac_user[0] = h.energy;
	mac_user[1] = h.entropy;
	mac_user[2] = h.correl;
	mac_user[3] = h.id;
	mac_user[4] = h.idm;
	mac_user[5] = h.contrast;
	mac_user[6] = h.clustering;
	mac_user[7] = h.inertia;
	

	return r;
}




/*      #     #   #   #####    #*#     #*#     #*#    ####    ####    #####   #      */
/*     # #    #   #     #     #   #   #   #   #   #   #   #   #   #   #       #      */
/*    #   #   #   #     #     #   #   #       #   #   #   #   #   #   #       #      */
/*    #   #   #   #     #     #   #   #       #   #   ####    ####    ####    #      */
/*    #####   #   #     #     #   #   #       #   #   # #     # #     #       #      */
/*    #   #   #   #     #     #   #   #   #   #   #   #  #    #  #    #       #      */
/*    #   #    #*#      #      #*#     #*#     #*#    #   #   #   #   #####   #####  */


/* This function computes the autocorrelation function of the 2D input image.
	Only positive displacements are used since the autocorrelation function
	is symmetric around the origin. The maximum displacement is the image size
	so the output image has the same size as the input image.*/


void comp_autocorrel (image_type *img, int slice)
{
image_type acm;		/* The result autocorrelation matrix */
float *p;
float buf, sum1, sum2;
int x,y,xm,ym,yh,dx,dy,x1,y1;
int bx1,bx2,by1,by2;			/* ROI's bounding box */


	xm=img->xmax;
	ym=img->ymax; yh=ym/2;
	
	if (allocate_image (&acm, B_FLOAT, xm, ym, 1)==-1)
		fatal_memerr();
	p = acm.data;

	roi_bbox (img, &bx1, &by1, &bx2, &by2);			/* Observe ROI on input image */


	/* The acf requires us to go over the image in a 4-deep loop.
		This function is *EXTREMELY* slow!
		Of course, there is the convolution theorem that relates the acf
		to the image's Fourier transform, so maybe we should make use of this
		theorem for improved efficiency... */

	for (dy=0; dy<ym; dy++)							/* Y shift */
	{
		buf = (float)(dy)*100.0/ym;
		dp (2, "Autocorrelation: %5.1f percent complete\n",buf);
		for (dx=0; dx<xm; dx++)							/* X shift */
		{
			sum1=0.0; sum2=0.0;
			
			for (y=by1; y<=by2; y++)
				for (x=bx1; x<=bx2; x++)
				{
					if (in_roi(x,y))					/* Observe the bounding box if defined */
					{
						x1=x+dx; y1=y+dy;
						while (x1>=img->xmax) x1-=img->xmax;		/* Roll-over */
						while (y1>=img->ymax) y1-=img->ymax;
						buf = readbuf_flt (img,x,y,slice);
						sum1 += buf*readbuf_flt (img,x1,y1,slice);
						sum2 += SQR(buf);
					}
				}
			if (F_ISZERO(sum2)) sum1=0; else sum1/=sum2;
			p[dx + acm.xmax*(dy)] = sum1;
		}
	}

	make_minmax (&acm);
	handback (img, &acm);
}


void autocorrel_main ()
{
	comp_autocorrel (&mainimg, cur_slice);
	cur_slice=0;
	make_imlib_buffer_from_main_image (cur_slice);
}




/*
	ACF function:
	Syntax: imgvar = ACF (imgexpr, slice);
	Create the autocorrelation function of an image (single-slice operation on slice "slice")

*/

variable_type mac_acf (unsigned char* s, int* i)
{
variable_type v,r;
int j,k;
int slice, numbins, xshift, yshift;


	/* The image stack */

	j = *i; k=0; v = nullvar;
	v = expr (s,&j, MT_IMG);		/* Get image to compute GLCM of */
	
	/* The parameters (all are integer) */	

	if (s[j++]!=',') macro_error (1,j);
	r = expr (s,&j, MT_INT); slice = r.intvalue;

	while (s[j]!=')') j++;
	*i = j+1;

	/* Go execute. Prepare v to be returned as image */

	comp_autocorrel (&v.imgptr, slice);
	v.type=MT_IMG;

	return v;

}



/**************************************************************

	This is the infrastructure we use to register with the 
	main program. The pointer "menufunction" points to the 
	main entry (the function to be called when the menu is 
	activated). Initmodule is the one function that the 
	main program calls (the name is fixed!) to initialize
	the module and to register the module with the menu.

***************************************************************/

void (*menufunction1)();
void (*menufunction2)();


variable_type (*macrofunction1)(unsigned char* s, int* i);
variable_type (*macrofunction2)(unsigned char* s, int* i);
variable_type (*macrofunction3)(unsigned char* s, int* i);


void module_cooc_fn1 ()
{

	if (file_ok) 
	{
		cooc_main ();

	}
}

void module_acf_fn1 ()
{

	if (file_ok) 
	{
		autocorrel_main ();

	}
}




void initmodule ()
{


	dp (1, "Co-occurrence matrix module: Initializer called\n");
	
	/* Register this module with the main menu */

	menufunction1 = &module_cooc_fn1;
	menufunction2 = &module_acf_fn1;
	
	dso_menu_register ("Co-occurrence matrix", menufunction1);
	dso_menu_register ("Autocorrelation matrix", menufunction2);

	/* Register the module with the macro interpreter.
		Note: Order of definition matters */
	
	macrofunction2 = &mac_glcmvector;
	register_dso_macro ("GLCMVECTOR",1,MT_REAL, macrofunction2);
	macrofunction1 = &mac_glcm;
	register_dso_macro ("GLCM",4,MT_IMG, macrofunction1);
	macrofunction3 = &mac_acf;
	register_dso_macro ("ACF",2,MT_IMG, macrofunction3);


}

/********************************************************************************/
